package tools.descartes.dni.dnimm3.tools;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.junit.Assert;
import org.junit.Test;

import tools.descartes.dni.dnimm3ap.AdaptationPoints;
import tools.descartes.dni.dnimm3ap.impl.AdaptationPointsImpl;
import tools.descartes.dni.dnimm3ap.impl.DNIAPFactoryImpl;
import tools.descartes.dni.dnimm3ap.tools.DniApParser;

public class ParserTest {

	@Test
	public void shouldConvertFileToModel() {
		File file = new File("resources-test/scenario0B.dniap");
		AdaptationPointsImpl adaptationPoints = DniApParser.convertToModel(file);
		Assert.assertTrue(adaptationPoints.getRepositories().getNodes().size() > 0);
	}

	@Test
	public void shouldConvertFileToModelWithNetworkStructure() {
		File file = new File("resources-test/scenario0B.dniap");
		AdaptationPointsImpl adaptationPoints = DniApParser.convertToModel(file);
		Assert.assertEquals(6, adaptationPoints.getNetworkInfrastructure().getStructure().getNodes().size());
	}

	@Test
	public void shouldConvertModelToFile() throws IOException {
		DNIAPFactoryImpl factory = new DNIAPFactoryImpl();
		AdaptationPoints model = factory.createAdaptationPoints();
		File file = File.createTempFile("junit-dni-", ".dniap");
		file.deleteOnExit();
		DniApParser.writeToFile(model, file);
		Assert.assertTrue(file.length() > 100);
	}

	@Test
	public void shouldConvertModelToByteArray() throws IOException {
		DNIAPFactoryImpl factory = new DNIAPFactoryImpl();
		AdaptationPoints model = factory.createAdaptationPoints();
		byte[] bytes = DniApParser.convertToByteArray(model);
		Assert.assertTrue(bytes.length > 10);
	}

	@Test
	public void shouldConvertByteArrayToModel() throws IOException {
		Path path = Paths.get("resources-test/scenario0B.dniap");
		byte[] data = Files.readAllBytes(path);
		AdaptationPoints adaptationPoints = DniApParser.convertToModel(data);
		Assert.assertTrue(adaptationPoints.getRepositories().getNodes().size() > 0);
	}

	@Test
	public void shouldCloneModel() {
		File file = new File("resources-test/scenario0B.dniap");
		AdaptationPointsImpl adaptationPoints1 = DniApParser.convertToModel(file);
		AdaptationPointsImpl adaptationPoints2 = DniApParser.clone(adaptationPoints1);
		Assert.assertNotEquals(adaptationPoints1, adaptationPoints2);
		Assert.assertEquals(adaptationPoints1.getNetworkInfrastructure().getStructure().getNodes().size(),
				adaptationPoints2.getNetworkInfrastructure().getStructure().getNodes().size());
		Assert.assertEquals(
				adaptationPoints1.getNetworkInfrastructure().getStructure().getNodes().get(0).getUid_generated(),
				adaptationPoints2.getNetworkInfrastructure().getStructure().getNodes().get(0).getUid_generated());
	}

}
